@@ -42,12 +42,13 @@
  * terminal_option_cmp:
  * @long_name     : long option text or %NULL
  * @short_name    : short option character or 0
- * @argv          : pointer to the argument vector
  * @argc          : length of the argument vector
+ * @argv          : pointer to the argument vector
  * @argv_offset   : current offset in the argument vector
  * @return_string : return location of a pointer to the option
  *                  arguments, or %NULL to make this function
  *                  behave for string comparison.
+ * @short_offset  : current offset in grouped short arguments e.g. -abcd
  *
  * Return value: %TRUE if @long_name or @short_name was found.
  **/
@@ -57,10 +58,12 @@ terminal_option_cmp (const gchar   *long_name,
                      gint           argc,
                      gchar        **argv,
                      gint          *argv_offset,
-                     gchar        **return_string)
+                     gchar        **return_string,
+                     gint          *short_offset)
 {
   gint   len, offset;
   gchar *arg = argv[*argv_offset];
+  gboolean short_option = FALSE;
 
   if (long_name != NULL && *(arg + 1) == '-')
     {
@@ -74,12 +77,24 @@ terminal_option_cmp (const gchar   *long_name,
 
       offset = 2 + len;
     }
-  else if (short_name != 0 && *(arg + 1) == short_name)
+  else if (short_name != 0 && *(arg + 1 + *short_offset) == short_name)
     {
       if (return_string == NULL)
-        return (*(arg + 2) == '\0');
+        {
+          if (*(arg + 2 + *short_offset) != '\0')
+            {
+              ++*short_offset;
+              --*argv_offset;
+            }
+          else
+            *short_offset = 0;
 
-      offset = 2;
+          return TRUE;
+        }
+
+      offset = 2 + *short_offset;
+      *short_offset = 0;
+      short_option = TRUE;
     }
   else
     {
@@ -87,12 +102,14 @@ terminal_option_cmp (const gchar   *long_name,
     }
 
   terminal_assert (return_string != NULL);
-  if (*(arg + offset) == '=')
-    *return_string = arg + (offset + 1);
-  else if (*argv_offset + 1 > argc)
-    *return_string = NULL;
-  else
+  if (*(arg + offset) == '\0')
     *return_string = argv[++*argv_offset];
+  else if (short_option)
+    *return_string = arg + offset;
+  else if (*(arg + offset) == '=')
+    *return_string = arg + (offset + 1);
+  else
+    return FALSE;
 
   return TRUE;
 }
@@ -133,28 +150,48 @@ terminal_options_parse (gint              argc,
                         gchar           **argv,
                         TerminalOptions  *options)
 {
-  gint n;
+  gint n, short_offset = 0;
 
   for (n = 1; n < argc; ++n)
     {
-      /* all arguments should atleast start with a dash */
+      /* all arguments should at least start with a dash */
       if (argv[n] == NULL || *argv[n] != '-')
         continue;
 
-      /* everything after execute belongs to the command */
-      if (terminal_option_cmp ("execute", 'x', argc, argv, &n, NULL))
+      /* "--" option delimiter */
+      if (terminal_option_cmp ("", 0, argc, argv, &n, NULL, &short_offset))
         break;
 
-      if (terminal_option_cmp ("help", 'h', argc, argv, &n, NULL))
-        options->show_help = 1;
-      else if (terminal_option_cmp ("version", 'V', argc, argv, &n, NULL))
-        options->show_version = 1;
-      else if (terminal_option_cmp ("disable-server", 0, argc, argv, &n, NULL))
+      /* everything after execute belongs to the command */
+      if (terminal_option_cmp ("execute", 'x', argc, argv, &n, NULL, &short_offset))
+        break;
+
+      if (terminal_option_cmp ("help", 'h', argc, argv, &n, NULL, &short_offset))
+        {
+          options->show_help = 1;
+          break;
+        }
+
+      if (terminal_option_cmp ("version", 'V', argc, argv, &n, NULL, &short_offset))
+        {
+          options->show_version = 1;
+          break;
+        }
+
+      if (terminal_option_cmp ("color-table", 0, argc, argv, &n, NULL, &short_offset))
+        {
+          options->show_colors = 1;
+          break;
+        }
+
+      if (terminal_option_cmp ("preferences", 0, argc, argv, &n, NULL, &short_offset))
+        {
+          options->show_preferences = 1;
+          break;
+        }
+
+      if (terminal_option_cmp ("disable-server", 0, argc, argv, &n, NULL, &short_offset))
         options->disable_server = 1;
-      else if (terminal_option_cmp ("color-table", 0, argc, argv, &n, NULL))
-        options->show_colors = 1;
-      else if (terminal_option_cmp ("preferences", 0, argc, argv, &n, NULL))
-        options->show_preferences = 1;
     }
 }
 
@@ -162,16 +199,22 @@ terminal_options_parse (gint              argc,
 
 /**
  * terminal_window_attr_parse:
- * @argc            :
- * @argv            :
- * @error           :
+ * @argc                : argument count
+ * @argv                : arguments
+ * @can_reuse_window    : a %gboolean denoting whether new tabs should be opened in an existing window or not
+ * @error               : a %GError used for error reporting
  *
- * Return value: %NULL on failure.
+ * As far as "--tab" and "--window" are concerned: Each "--window" argument opens an additional window.
+ * Each "--tab" argument opens an additional tab in the preceding window (or in the current active window, if such exists,
+ * and no "--window" argument has been read previously).
+ *
+ *
+ * Return value: a list of %TerminalWindowAttr or %NULL on failure.
  **/
 GSList *
 terminal_window_attr_parse (gint              argc,
                             gchar           **argv,
-                            gboolean          can_reuse_tab,
+                            gboolean          can_reuse_window,
                             GError          **error)
 {
   TerminalWindowAttr *win_attr;
@@ -181,21 +224,25 @@ terminal_window_attr_parse (gint              argc,
   gchar              *s;
   GSList             *tp, *wp;
   GSList             *attrs;
-  gint                n;
+  gint                n, short_offset = 0;
   gchar              *end_ptr = NULL;
   TerminalVisibility  visible;
 
   win_attr = terminal_window_attr_new ();
   tab_attr = win_attr->tabs->data;
-  attrs = g_slist_prepend (NULL, win_attr);
+  attrs = g_slist_prepend (NULL, win_attr); /* used as the return value */
 
   for (n = 1; n < argc; ++n)
     {
-      /* all arguments should atleast start with a dash */
+      /* all arguments should at least start with a dash */
       if (argv[n] == NULL || *argv[n] != '-')
         goto unknown_option;
 
-      if (terminal_option_cmp ("default-display", 0, argc, argv, &n, &s))
+      /* "--" option delimiter*/
+      if (terminal_option_cmp ("", 0, argc, argv, &n, NULL, &short_offset))
+        break;
+
+      if (terminal_option_cmp ("default-display", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -211,7 +258,7 @@ terminal_window_attr_parse (gint              argc,
               continue;
             }
         }
-      else if (terminal_option_cmp ("default-working-directory", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("default-working-directory", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -228,13 +275,14 @@ terminal_window_attr_parse (gint              argc,
               continue;
             }
         }
-      else if (terminal_option_cmp ("execute", 'x', argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("execute", 'x', argc, argv, &n, NULL, &short_offset))
         {
-          if (++n >= argc)
+          if (short_offset > 0 || ++n >= argc)
             {
               g_set_error (error, G_SHELL_ERROR, G_SHELL_ERROR_FAILED,
                            _("Option \"--execute/-x\" requires specifying the command "
-                             "to run on the rest of the command line"));
+                             "to run on the rest of the command line separated from "
+                             "\"--execute/-x\""));
               goto failed;
             }
           else
@@ -245,7 +293,7 @@ terminal_window_attr_parse (gint              argc,
 
           break;
         }
-      else if (terminal_option_cmp ("command", 'e', argc, argv, &n, &s))
+      else if (terminal_option_cmp ("command", 'e', argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -262,7 +310,7 @@ terminal_window_attr_parse (gint              argc,
                 goto failed;
             }
         }
-      else if (terminal_option_cmp ("working-directory", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("working-directory", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -277,7 +325,7 @@ terminal_window_attr_parse (gint              argc,
               tab_attr->directory = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("title", 'T', argc, argv, &n, &s))
+      else if (terminal_option_cmp ("title", 'T', argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -292,7 +340,7 @@ terminal_window_attr_parse (gint              argc,
               tab_attr->title = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("dynamic-title-mode", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("dynamic-title-mode", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -317,7 +365,7 @@ terminal_window_attr_parse (gint              argc,
               goto failed;
             }
         }
-      else if (terminal_option_cmp ("initial-title", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("initial-title", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -332,15 +380,15 @@ terminal_window_attr_parse (gint              argc,
               tab_attr->initial_title = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("hold", 'H', argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("hold", 'H', argc, argv, &n, NULL, &short_offset))
         {
           tab_attr->hold = TRUE;
         }
-      else if (terminal_option_cmp ("active-tab", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("active-tab", 0, argc, argv, &n, NULL, &short_offset))
         {
           tab_attr->active = TRUE;
         }
-      else if (terminal_option_cmp ("color-text", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("color-text", 0, argc, argv, &n, &s, &short_offset))
         {
           GdkRGBA color;
           if (G_UNLIKELY (s == NULL))
@@ -359,7 +407,7 @@ terminal_window_attr_parse (gint              argc,
           g_free (tab_attr->color_text);
           tab_attr->color_text = g_strdup (s);
         }
-      else if (terminal_option_cmp ("color-bg", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("color-bg", 0, argc, argv, &n, &s, &short_offset))
         {
           GdkRGBA color;
           if (G_UNLIKELY (s == NULL))
@@ -378,7 +426,7 @@ terminal_window_attr_parse (gint              argc,
           g_free (tab_attr->color_bg);
           tab_attr->color_bg = g_strdup (s);
         }
-      else if (terminal_option_cmp ("display", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("display", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -393,7 +441,7 @@ terminal_window_attr_parse (gint              argc,
               win_attr->display = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("geometry", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("geometry", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -408,7 +456,7 @@ terminal_window_attr_parse (gint              argc,
               win_attr->geometry = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("role", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("role", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -423,7 +471,7 @@ terminal_window_attr_parse (gint              argc,
               win_attr->role = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("sm-client-id", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("sm-client-id", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -438,7 +486,7 @@ terminal_window_attr_parse (gint              argc,
               win_attr->sm_client_id = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("startup-id", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("startup-id", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -454,7 +502,7 @@ terminal_window_attr_parse (gint              argc,
               continue;
             }
         }
-      else if (terminal_option_cmp ("icon", 'I', argc, argv, &n, &s))
+      else if (terminal_option_cmp ("icon", 'I', argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -469,7 +517,7 @@ terminal_window_attr_parse (gint              argc,
               win_attr->icon = g_strdup (s);
             }
         }
-      else if (terminal_option_cmp ("drop-down", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("drop-down", 0, argc, argv, &n, NULL, &short_offset))
         {
           win_attr->drop_down = TRUE;
         }
@@ -477,15 +525,15 @@ terminal_window_attr_parse (gint              argc,
         {
           win_attr->menubar = visible;
         }
-      else if (terminal_option_cmp ("fullscreen", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("fullscreen", 0, argc, argv, &n, NULL, &short_offset))
         {
           win_attr->fullscreen = TRUE;
         }
-      else if (terminal_option_cmp ("maximize", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("maximize", 0, argc, argv, &n, NULL, &short_offset))
         {
           win_attr->maximize = TRUE;
         }
-      else if (terminal_option_cmp ("minimize", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("minimize", 0, argc, argv, &n, NULL, &short_offset))
         {
           win_attr->minimize = TRUE;
         }
@@ -501,12 +549,13 @@ terminal_window_attr_parse (gint              argc,
         {
           win_attr->scrollbar = visible;
         }
-      else if (terminal_option_cmp ("tab", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("tab", 0, argc, argv, &n, NULL, &short_offset))
         {
-          if (can_reuse_tab)
+          if (can_reuse_window)
             {
-              /* tab is the first user option, reuse existing window */
+              /* add tabs in an existing active window, until a '--window' option in encountered */
               win_attr->reuse_last_window = TRUE;
+              can_reuse_window = FALSE;
             }
           else
             {
@@ -515,17 +564,17 @@ terminal_window_attr_parse (gint              argc,
               win_attr->tabs = g_slist_append (win_attr->tabs, tab_attr);
             }
         }
-      else if (terminal_option_cmp ("window", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("window", 0, argc, argv, &n, NULL, &short_offset))
         {
-          /* multiple windows, don't reuse */
-          win_attr->reuse_last_window = FALSE;
+          /* all new tabs will be added to new windows */
+          can_reuse_window = FALSE;
 
           /* setup new window */
           win_attr = terminal_window_attr_new ();
           tab_attr = win_attr->tabs->data;
           attrs = g_slist_append (attrs, win_attr);
         }
-      else if (terminal_option_cmp ("font", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("font", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL))
             {
@@ -541,7 +590,7 @@ terminal_window_attr_parse (gint              argc,
               continue;
             }
         }
-      else if (terminal_option_cmp ("zoom", 0, argc, argv, &n, &s))
+      else if (terminal_option_cmp ("zoom", 0, argc, argv, &n, &s, &short_offset))
         {
           if (G_UNLIKELY (s == NULL) ||
               strtol (s, &end_ptr, 0) < TERMINAL_ZOOM_LEVEL_MINIMUM ||
@@ -559,9 +608,9 @@ terminal_window_attr_parse (gint              argc,
               continue;
             }
         }
-      else if (terminal_option_cmp ("disable-server", 0, argc, argv, &n, NULL)
-               || terminal_option_cmp ("sync", 0, argc, argv, &n, NULL)
-               || terminal_option_cmp ("g-fatal-warnings", 0, argc, argv, &n, NULL))
+      else if (terminal_option_cmp ("disable-server", 0, argc, argv, &n, NULL, &short_offset)
+               || terminal_option_cmp ("sync", 0, argc, argv, &n, NULL, &short_offset)
+               || terminal_option_cmp ("g-fatal-warnings", 0, argc, argv, &n, NULL, &short_offset))
         {
           /* options we can ignore */
           continue;
@@ -569,13 +618,15 @@ terminal_window_attr_parse (gint              argc,
       else
         {
 unknown_option:
-          g_set_error (error, G_SHELL_ERROR, G_SHELL_ERROR_FAILED,
-                       _("Unknown option \"%s\""), argv[n]);
+          if (short_offset > 0)
+            g_set_error (error, G_SHELL_ERROR, G_SHELL_ERROR_FAILED,
+                         _("Unknown option \"-%c\""), *(argv[n] + 1 + short_offset));
+          else
+            g_set_error (error, G_SHELL_ERROR, G_SHELL_ERROR_FAILED,
+                         _("Unknown option \"%s\""), argv[n]);
+
           goto failed;
         }
-
-      /* not the first option anymore */
-      can_reuse_tab = FALSE;
     }
 
   /* substitute default working directory and default display if any */
